@@ -1,5 +1,3 @@
-from __future__ import print_function
-
 import json
 import os
 import re
@@ -7,23 +5,18 @@ import sys
 import tarfile
 import textwrap
 from contextlib import closing
+from urllib.request import urlretrieve
 
 from autocorrect.constants import word_regexes, urls
 from autocorrect.typos import Word
 
-if sys.version_info[0] == 3:
-    from urllib.request import urlretrieve
-else:
-    print("WARNING: python2 doesn't support correction with special chars")
-    from urllib import urlretrieve
-
 
 PATH = os.path.abspath(os.path.dirname(__file__))
 languages_url = "https://github.com/fsondej/autocorrect/raw/master/\
 optional_languages/{}.tar.gz"
 
 
-# credit: https://stackoverflow.com/questions/43370284/why-function-works-properly-without-specifying-parameters
+# credit: https://stackoverflow.com/questions/43370284
 class ProgressBar:
     def __init__(self):
         self.old_percent = 0
@@ -41,22 +34,22 @@ class ProgressBar:
 
 
 def load_from_tar(lang, file_name='word_count.json'):
-    archive_name = os.path.join(PATH, 'data/{}.tar.gz'.format(lang))
+    archive_name = os.path.join(PATH, f"data/{lang}.tar.gz")
 
     if lang not in word_regexes:
         supported_langs = ', '.join(word_regexes.keys())
         raise NotImplementedError(
-            textwrap.dedent("""
-            language '{}' not supported
-            supported languages: {}
+            textwrap.dedent(f"""
+            language '{lang}' not supported
+            supported languages: {supported_langs}
             you can easily add new languages by following instructions at
             https://github.com/fsondej/autocorrect/tree/master#adding-new-languages
-            """.format(lang, supported_langs)))
+            """))
 
     if not os.path.isfile(archive_name):
         print('dictionary for this language not found, downloading...')
         possible_urls = [
-            urls[lang], 
+            urls[lang],
             # languages_url.format(lang)
         ]
         for url in possible_urls:
@@ -66,10 +59,11 @@ def load_from_tar(lang, file_name='word_count.json'):
                 error_message = None
                 break
             except Exception as ex:
-                print("couldn't download {}, trying next url...".format(url))
+                print(f"couldn't download {url}, trying next url...")
                 error_message = str(ex)
         if error_message is not None:
-            raise ConnectionError(error_message + \
+            raise ConnectionError(
+                error_message +
                 '\nFix your network connection, or manually download \n{}'
                 '\nand put it in \nPATH_TO_REPO/autocorrect/data/'.format(url))
 
@@ -86,20 +80,21 @@ class Speller:
         self.fast = fast
 
         if threshold > 0:
-            print('Original number of words: {}'
-                  .format(len(self.nlp_data)))
+            print(f'Original number of words: {len(self.nlp_data)}')
             self.nlp_data = {k: v for k, v in self.nlp_data.items()
                              if v >= threshold}
-            print('After applying threshold: {}'
-                  .format(len(self.nlp_data)))
+            print(f'After applying threshold: {len(self.nlp_data)}')
 
     def existing(self, words):
         """{'the', 'teh'} => {'the'}"""
-        return set(word for word in words
-                   if word in self.nlp_data)
+        return {word for word in words
+                if word in self.nlp_data}
 
     def autocorrect_word(self, word):
         """most likely correction for everything up to a double typo"""
+        if word == '':
+            return ''
+
         def get_candidates(word):
             w = Word(word, self.lang)
             if self.fast:
